# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/11/2019"

from PyMca5.PyMcaPhysics.xas.XASClass import XASClass
from xas.core.types import XASObject, Spectrum
from .progress import Progress
import logging

_logger = logging.getLogger(__name__)


def _process_spectrum(spectrum, configuration):
    pymca_xas = XASClass()
    if spectrum.energy is None or spectrum.mu is None:
        _logger.error('Energy and or Mu is/are not specified, unable to '
                      'compute exafs')
        return None, None
    pymca_xas.setConfiguration(configuration)
    pymca_xas.setSpectrum(energy=spectrum.energy,
                          mu=spectrum.mu)
    spectrum = Spectrum.from_dict(pymca_xas.processSpectrum())
    assert 'FT' in spectrum
    return configuration, spectrum


def pymca_ft(xas_obj):
    """

    :param xas_obj: object containing the configuration and spectra to process
    :type: Union[XASObject, dict]
    :return: spectra dict
    :rtype: dict
    """
    ft_obj = PyMca_ft()
    return ft_obj.process(xas_obj=xas_obj)


class PyMca_ft(object):
    def __init__(self):
        self._settings = None
        self._advancement = Progress('ft')

    def setProperties(self, properties):
        if '_pymcaSettings' in properties:
            self._settings = properties['_pymcaSettings']

    def process(self, xas_obj):
        """
    
        :param xas_obj: object containing the configuration and spectra to process
        :type: Union[XASObject, dict]
        :return: spectra dict
        :rtype: dict
        """
        if isinstance(xas_obj, dict):
            _xas_obj = XASObject.from_dict(xas_obj)
        else:
            _xas_obj = xas_obj
        if self._settings:
            _xas_obj.configuration['FT'] = self._settings

        self._advancement.startProcess()
        for i_spectrum, spectrum in enumerate(_xas_obj.spectra):
            _xas_obj.configuration, _xas_obj.spectra[
                i_spectrum] = _process_spectrum(spectrum=spectrum,
                                                configuration=_xas_obj.configuration)
            self._advancement.setAdvancement(int((i_spectrum * _xas_obj.n_spectrum) * 100))
        self._advancement.endProcess()
        return _xas_obj

    __call__ = process
