# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2017-2019 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/11/2019"


import unittest
from xas.io import read_pymca_xas
from silx.io.url import DataUrl
from PyMca5.PyMcaDataDir import PYMCA_DATA_DIR
from xas.core.types import XASObject
import shutil
import tempfile
import numpy
import os


class TestReadWrite(unittest.TestCase):
    """Test read function for spectra and configuration"""
    def setUp(self):
        self.outputdir = tempfile.mkdtemp()
        self.outputfile = os.path.join(self.outputdir, 'output_file.h5')

    def tearDown(self):
        shutil.rmtree(self.outputdir)

    def testReadSpectrum(self):
        data_file = os.path.join(PYMCA_DATA_DIR, "EXAFS_Cu.dat")
        res = read_pymca_xas(spectra_url=DataUrl(file_path=data_file,
                                                 scheme='PyMca'),
                             channel_url=DataUrl(file_path=data_file,
                                                 scheme='PyMca'))
        self.assertTrue(isinstance(res, XASObject))
        self.assertEqual(res.n_spectrum, 1)
        self.assertTrue('Mu' in res.spectra[0])
        self.assertTrue('Energy' in res.spectra[0])

    def testReadWriteH5(self):
        """Tset io relative to h5py"""
        xas_obj = XASObject(configuration={'axis': 2},
                            spectra=numpy.arange(10).reshape(10, 1, 1),
                            energy=numpy.arange(10))

        xas_obj.dump(self.outputfile)
        self.assertTrue(os.path.exists(self.outputfile))
        xas_obj2 = XASObject.from_file(self.outputfile)
        assert xas_obj2
        self.assertEqual(xas_obj, xas_obj2)


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestReadWrite,):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == '__main__':
    unittest.main(defaultTest="suite")
