# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2017-2019 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/11/2019"


import unittest
import os
from PyMca5.PyMcaDataDir import PYMCA_DATA_DIR
from silx.io.url import DataUrl
from xas.core.process.exafs import pymca_exafs, PyMca_exafs
from xas.core.process.ft import pymca_ft, PyMca_ft
from xas.io import read_pymca_xas
from xas.core.process.k_weight import pymca_k_weight, PyMca_k_weight
from xas.core.process.normalization import pymca_normalization, PyMca_normalization


class TestStreamSingleSpectrum(unittest.TestCase):
    """Make sure the process have valid io"""
    def test_pymca_process(self):
        data_file = os.path.join(PYMCA_DATA_DIR, "EXAFS_Cu.dat")
        out = read_pymca_xas(spectra_url=DataUrl(file_path=data_file,
                                                 scheme='PyMca'),
                             channel_url=DataUrl(file_path=data_file,
                                                 scheme='PyMca'))
        out = pymca_normalization(xas_obj=out)
        out = pymca_k_weight(xas_obj=out)
        out = pymca_exafs(xas_obj=out)
        out = pymca_ft(xas_obj=out)
        assert out is not None

    def test_pymca_process_with_cls(self):
        data_file = os.path.join(PYMCA_DATA_DIR, "EXAFS_Cu.dat")
        out = read_pymca_xas(spectra_url=DataUrl(file_path=data_file,
                                                 scheme='PyMca'),
                             channel_url=DataUrl(file_path=data_file,
                                                 scheme='PyMca'))
        out = PyMca_normalization()(xas_obj=out)
        out = PyMca_k_weight()(xas_obj=out)
        out = PyMca_exafs()(xas_obj=out)
        out = PyMca_ft()(xas_obj=out)
        assert out is not None


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestStreamSingleSpectrum,):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == '__main__':
    unittest.main(defaultTest="suite")
